/**
 * Sanitize function
 */
function sanitizeCodeblock(str) {
    if ( str ) {
        return str.replace(/</g, '&lt;').replace(/>/g, '&gt;').replace(/"/g, '&quot;').replace(/'/g, '&#39;');
    } else {
        return str;
    }
}

/**
 * Function for setting code block height
 */
function setHeightCodeBlocks( elem ) {
    var num = elem.value.match(/\r\n|\n/g);
    if (num != null) {
        var line = num.length + 1;
    } else {
        var line = 1;
    }
    elem.style.height = line * 1.8 + 1.6 + 2 + "em";
}

/**
 * Valid Blocks - 高度な設定を追加するブロックを指定
 */
function isValidBlockType(blockName) {

    var validBlocks = [
        "loos-hcb/code-block",
    ]
    return validBlocks.indexOf(blockName) !== -1;
}

/**
 * Add Custom Block & Controls
 */
(function () {
    //グローバル変数の受け取り
    var global_hcb_langs = hcbLangArray;
    if (typeof global_hcb_langs != "object") {
        global_hcb_langs = {};
        alert('エラー：「Highlighting Code Block」プラグインの「使用する言語設定」を見直してください。');
    }

    // wp->
    var editor     = wp.editor,
        blocks     = wp.blocks,
        element    = wp.element,
        i18n       = wp.i18n,
        compose    = wp.compose,
        components = wp.components,
        addFilter  = wp.hooks.addFilter,
        el         = element.createElement,
        Fragment   = element.Fragment,
        RichText   = editor.RichText,
        InspectorControls          = editor.InspectorControls;
        createHigherOrderComponent = compose.createHigherOrderComponent;

 
    /**
     * Register Highlighting Code Block
     */
    blocks.registerBlockType('loos-hcb/code-block', {
        title: 'Highlighing Code Block',
        icon: 'editor-code',
        category: 'formatting',
        attributes: {
            code: {
                type: 'string',
                source: 'text',
                selector: 'code',
            },
            className: {
                type: 'string',
            },
            langType: {
                type: 'string',
            },
            langName: {
                type: 'string',
            },
            dataLineNum: {
                type: 'text',
            },
            isLineShow: {
                type: 'string',
            },
            preClass: {
                type: 'string',
            },
        },

        edit: function (props) {
            var attributes  = props.attributes;
            var hcbElemList = [];
            var optionsList = [
                el('option', { value: '' }, '- Lang Select -'),
            ];

            Object.keys(global_hcb_langs).forEach(function (key) {
                optionsList.push(
                    el('option', { value: key }, global_hcb_langs[key])
                );
            });
            if (!attributes.className) {
                props.setAttributes({ className: 'hcb_wrap' });
            }
            if (!attributes.langType) {
                props.setAttributes({ langType: 'plane' });
            }
            if (!attributes.isLineShow) {
                props.setAttributes({ isLineShow: 'undefined' });
            }

            var preClass = 'prism ' + attributes.isLineShow + '-numbers lang-' + attributes.langType;
            props.setAttributes({ preClass: preClass });

            hcbElemList.push(
                el('textarea', {
                    className: "hcb_textarea",
                    placeholder: 'Your Code...',
                    value: attributes.code,
                    onChange: function (event) {
                        props.setAttributes({ code: event.target.value });
                        setHeightCodeBlocks(event.target);
                        event.preventDefault();
                    },
                })
            );

            hcbElemList.push(
                el('div', {className: 'select_wrap'}, 
                    el('select', { 
                        value: attributes.langType,
                        onChange: function (event) {
                            var selected = event.target.querySelector('option:checked');
                            var selectedLangName = selected.text;
                            if (selectedLangName === "- Lang Select -") {
                                selectedLangName = '';
                            }
                            props.setAttributes({ langType: selected.value });
                            props.setAttributes({ langName: selectedLangName });
                            event.preventDefault();
                        }
                    }, optionsList ),
                    el('input', { 
                        type: 'text',
                        className: "num_input",
                        value: attributes.dataLineNum,
                        placeholder: 'data-line属性の値',
                        onChange: function (event) {
                            props.setAttributes({ dataLineNum: event.target.value });
                            event.preventDefault();
                        }
                    }),
                )
            );
            return el('div', { className: attributes.className }, hcbElemList );
        },
        save: function (props) {
            var attributes = props.attributes;

            return (
                el('div', { 
                    className: attributes.className,
                 }, el('pre', {
                        className: attributes.preClass,
                        "data-lang": attributes.langName,
                        "data-line": attributes.dataLineNum,
                    }, el(RichText.Content, {
                            tagName: 'code', value: sanitizeCodeblock( attributes.code )
                        })
                    )
                )
            );
        },
    });


    /**
     * Add Higher Controls
     */
    var addMyControls = createHigherOrderComponent(function (BlockEdit) {
        return function (props) {

            var isLineShow = props.attributes.isLineShow;
            if (typeof isLineShow === "undefined") {
                isLineShow = "undefined";
            }

            var myComponent = el(components.PanelBody, {
                title: "HCB設定"
            }, el(components.RadioControl, {
                label: "このブロックの行数表示",
                //help: "help text",
                selected: isLineShow,
                options: [{
                    label: "個別設定はしない",
                    value: "undefined"
                }, {
                    label: "行数を表示する",
                    value: "on"
                }, {
                    label: "行数を表示しない",
                    value: "off"
                }],
                onChange: function (val) {
                    props.setAttributes({ isLineShow: val })
                }
            }),
            );

            return isValidBlockType(props.name) && props.isSelected ? el(Fragment, null, el(BlockEdit, props), el(InspectorControls, null, myComponent)) : el(BlockEdit, props)
        }
    }, "addMyControls");


    /**
     * Override props assigned to save component to inject our custom data.
     * This is only done if the component is a valid block type.
     *
     * @param {Object} extraProps Additional props applied to save element.
     * @param {Object} blockType  Block type.
     * @param {Object} attributes Current block attributes.
     *
     * @return {Object} Filtered props applied to save element.
     */
    function addSaveProps(extraProps, blockType, attributes) {
        
        if (isValidBlockType(blockType.name)) {
                // 「wp-block-loos-hcb-code-block」をつけない
            extraProps.className = attributes.className;
        }
        return extraProps;

    }


    /**
     * ベータ版での設定値によるバリデーション回避
     * @param {Object} attributes Current block attributes.
     * @param {Object} settings Current block settings.
     * @param {Object} content Current block content.
     */
    function beforeValidation(attributes, settings, content) {

        //console.log(settings.name);
        if (settings.name === 'loos-hcb/code-block') {

            if (typeof attributes.preClass === "undefined") {
                var isLineShow = attributes.isLineShow;
                if (typeof isLineShow === "undefined") {
                    isLineShow = "line"; // "on" or "off"
                }
                var preClass = 'prism ' + isLineShow + '-numbers lang-' + attributes.langType;
                attributes.preClass = preClass;
            }

        }
        return attributes;

    };

    addFilter("editor.BlockEdit", "loos-hcb/my-control", addMyControls);
    addFilter("blocks.getBlockAttributes", "loos-hcb/before-validation", beforeValidation);
    addFilter("blocks.getSaveContent.extraProps", "loos-hcb/add-props", addSaveProps);

})();


/**
 * DOMContentLoaded Event
 */
document.addEventListener('DOMContentLoaded', function () {
    /* コードブロック　高さセット */
    var codeBlocks = document.querySelectorAll('.hcb_textarea');
    for (var index = 0; index < codeBlocks.length; index++) {
        var element = codeBlocks[index];
        setHeightCodeBlocks( element );
    }
});
